@@ -19,12 +19,14 @@ if config.dark_mode:
     UI_HIGHLIGHT_FG = FG
     UI_HIGHLIGHT_BG = 242
     UI_ALT_HIGHLIGHT_BG = 203
+    UI_ERROR_FG = 196
 else:
     FG = 232
     BG = 231
     UI_HIGHLIGHT_FG = FG
     UI_HIGHLIGHT_BG = 250
     UI_ALT_HIGHLIGHT_BG = 203
+    UI_ERROR_FG = 196
 
 # ANSI color code for each mine value
 if config.dark_mode:
@@ -32,14 +34,15 @@ if config.dark_mode:
 else:
     VALUES = [253, 21, 41, 160, 165, 208, 69, 92, 239]
 
-UI_COLORS_USED = [FG, BG, UI_HIGHLIGHT_FG, UI_HIGHLIGHT_BG, UI_ALT_HIGHLIGHT_BG]
+UI_COLORS_USED = [FG, BG, UI_HIGHLIGHT_FG, UI_HIGHLIGHT_BG, UI_ALT_HIGHLIGHT_BG, UI_ERROR_FG]
 UI_COLORS_USED.extend(VALUES)
 UI_COLORS_USED = sorted(list(set(UI_COLORS_USED)))  # remove duplicate and sort
 
 DEFAULT = 1
 UI_HIGHLIGHT = 2
 UI_ALT_HIGHLIGHT = 3
-SYSTEM_DEFAULT = 127
+UI_ERROR = 4
+SYSTEM_DEFAULT = 5
 
 # a simple wrapper around the mouse events for easier bitmask processing
 MouseEvent = IntFlag('MouseEvent',
@@ -58,8 +61,10 @@ def pad_window(line, width, center = False):
 
 def cell_color(value, highlight):
     """Calculates the color index of the given cell value and highlight state"""
-    return (value << 4) | (highlight << 3) | 0b111
+
+    # 3 bits = 8 colors are reserved for the UI
     # at value = 0, highlight = 0 this will start from 7
+    return (value << 4) | (highlight << 3) | 0b111
 
 
 def debug_print(*args, **kwargs):
@@ -383,35 +388,35 @@ class GridWidget(Widget):
 
                 # special case for first column
                 if not x:
-                    self.addstr(y * 2 + 1, 0, box(up=tl,down=tl))
-                    self.addstr(y * 2 + 3, 0, box(up=bl,down=bl))
-                    self.addstr(y * 2 + 2, 0, box(up=tl,down=bl,right=tl or bl))
+                    self.addstr(y * 2 + 1, 0, box(up = tl, down = tl))
+                    self.addstr(y * 2 + 3, 0, box(up = bl, down = bl))
+                    self.addstr(y * 2 + 2, 0, box(up = tl, down = bl, right = tl or bl))
 
                 # special case for last column
                 if x == self.board.width - 2:
-                    self.addstr(y * 2 + 1, x * 5 + 10, box(up=tr,down=tr))
-                    self.addstr(y * 2 + 3, x * 5 + 10, box(up=br,down=br))
-                    self.addstr(y * 2 + 2, x * 5 + 10, box(up=tr,down=br,left=tr or br))
+                    self.addstr(y * 2 + 1, x * 5 + 10, box(up = tr, down = tr))
+                    self.addstr(y * 2 + 3, x * 5 + 10, box(up = br, down = br))
+                    self.addstr(y * 2 + 2, x * 5 + 10, box(up = tr, down = br, left = tr or br))
 
                 # special case for first row
                 if not y:
-                    self.addstr(y * 2, x * 5 + 1, box(left=tl,right=tl) * 4)
-                    self.addstr(y * 2, x * 5 + 6, box(left=tr,right=tr) * 4)
-                    self.addstr(y * 2, x * 5 + 5, box(left=tl,right=tr,down=tl or tr))
+                    self.addstr(y * 2, x * 5 + 1, box(left = tl, right = tl) * 4)
+                    self.addstr(y * 2, x * 5 + 6, box(left = tr, right = tr) * 4)
+                    self.addstr(y * 2, x * 5 + 5, box(left = tl, right = tr, down = tl or tr))
 
                 # special case for last row
                 if y == self.board.height - 2:
-                    self.addstr(y * 2 + 4, x * 5 + 1, box(left=bl,right=bl) * 4)
-                    self.addstr(y * 2 + 4, x * 5 + 6, box(left=br,right=br) * 4)
-                    self.addstr(y * 2 + 4, x * 5 + 5, box(left=bl,right=br,up=bl or br))
+                    self.addstr(y * 2 + 4, x * 5 + 1, box(left = bl, right = bl) * 4)
+                    self.addstr(y * 2 + 4, x * 5 + 6, box(left = br, right = br) * 4)
+                    self.addstr(y * 2 + 4, x * 5 + 5, box(left = bl, right = br, up = bl or br))
 
                 # horizontal lines
-                self.addstr(y * 2 + 2, x * 5 + 1, box(left=tl or bl,right=tl or bl) * 4)
-                self.addstr(y * 2 + 2, x * 5 + 6, box(left=tr or br,right=tr or br) * 4)
+                self.addstr(y * 2 + 2, x * 5 + 1, box(left = tl or bl, right = tl or bl) * 4)
+                self.addstr(y * 2 + 2, x * 5 + 6, box(left = tr or br, right = tr or br) * 4)
 
                 # vertical lines
-                self.addstr(y * 2 + 1, x * 5 + 5, box(up=tl or tr,down=tl or tr))
-                self.addstr(y * 2 + 3, x * 5 + 5, box(up=bl or br,down=bl or br))
+                self.addstr(y * 2 + 1, x * 5 + 5, box(up = tl or tr, down = tl or tr))
+                self.addstr(y * 2 + 3, x * 5 + 5, box(up = bl or br, down = bl or br))
 
                 # the center of the cluster
                 self.addstr(y * 2 + 2, x * 5 + 5, box(tl or tr, bl or br, tl or bl, tr or br))
@@ -422,10 +427,10 @@ class GridWidget(Widget):
         br = not self.board[-1][-1].is_revealed  # bottom right
 
         # add the corners of the board
-        self.addstr(0, 0, box(right=tl,down=tl))
-        self.addstr(0, width - 1, box(left=tr,down=tr))
-        self.addstr(height - 1, 0, box(up=bl,right=bl))
-        self.addstr(height - 1, width - 1, box(up=br,left=br))
+        self.addstr(0, 0, box(right = tl, down = tl))
+        self.addstr(0, width - 1, box(left = tr, down = tr))
+        self.addstr(height - 1, 0, box(up = bl, right = bl))
+        self.addstr(height - 1, width - 1, box(up = br, left = br))
 
         if self.root.keyboard_mode:
             self.subwidgets[self.selected_cell].highlight(True)
@@ -543,8 +548,11 @@ class HelpWidget(Widget):
     @property
     def w(self):
         if not self.is_active:
-            # +1 for the full width character ⓘ
-            return len("ⓘ press i for help") + 1
+            if not config.use_emojis:
+                return len("press i for help")
+            else:
+                # +1 for the full width character ⓘ
+                return len("ⓘ press i for help") + 1
         else:
             return 54
 
@@ -563,7 +571,11 @@ class HelpWidget(Widget):
 
     def render(self):
         if not self.is_active:
-            self.addstr(0, 0, "ⓘ press i for help")
+            if not config.use_emojis:
+                # some terminals, e.g. terminus, can't render ⓘ
+                self.addstr(0, 0, "ⓘ press i for help")
+            else:
+                self.addstr(0, 0, "press i for help")
         else:
             mouse_y = self.mouse_y()
             mouse_x = self.mouse_x()
@@ -596,6 +608,8 @@ class HelpWidget(Widget):
             self.parent.force_rerender = True
 
     def mouse_event(self, y, x, mouse):
+        if not config.use_emojis:
+            return
         if MouseEvent.BUTTON1_RELEASED in mouse:
             if not self.is_active:
                 if y == 0 and x == 0:
@@ -750,10 +764,15 @@ class RootWidget(Widget):
         """
         all_events_processed = False
         while not all_events_processed:
+            winh, winw = self.window.getmaxyx()
             ch = self.window.getch()
-
+            if ch == curses.KEY_RESIZE:
+                char = '\0'
+                self.force_rerender = True
+                etype = 'other'
+                args = (char,)
             # handles mouse input
-            if ch == curses.KEY_MOUSE:
+            elif ch == curses.KEY_MOUSE:
                 try:
                     _, mouse_x, mouse_y, z, mouse_button = curses.getmouse()
                     mouse = MouseEvent(mouse_button)
@@ -809,15 +828,8 @@ class RootWidget(Widget):
                 etype = 'mouse'
                 args = (self.mouse_y, self.mouse_x, mouse)
             else:
-                # check resize
-                if ch == curses.KEY_RESIZE:
-                    winh, winw = self.window.getmaxyx()
-                    if not config.ignore_failures and (winh < config.min_height or winw < config.min_width):
-                        raise InsufficientScreenSpace
-                    char = '\0'
-                    self.force_rerender = True
                 # keyboard input
-                elif ch == -1 or self.button2_pressed:
+                if ch == -1 or self.button2_pressed:
                     # block middle button paste on linux
                     char = '\0'
                     all_events_processed = True
@@ -830,10 +842,15 @@ class RootWidget(Widget):
                 args = (char,)
 
             try:
+                if winh < config.min_height or winw < config.min_width:
+                    # ignore all events until size is fixed
+                    continue
                 if etype == 'keyboard' and char != '\0':
                     self.dispatch_event('keyboard', *args)
                 elif etype == 'mouse':
                     self.dispatch_event('mouse', *args)
+                elif etype == "other":
+                    pass
                 elif not self.keyboard_mode:
                     # hover
                     self.dispatch_event('mouse', self.mouse_y, self.mouse_x, MouseEvent(0))
@@ -847,48 +864,61 @@ class RootWidget(Widget):
         # caps the framerate by postponing rendering
         if config.framerate:
             self.render()
-            curses.napms(floor((1/config.framerate-(time.time()-self.monitor.last_frame))*1000))
+            curses.napms(floor((1 / config.framerate - (time.time() - self.monitor.last_frame)) * 1000))
         else:
             self.render()
-            curses.napms(1) # voluntary context switch
+            curses.napms(1)  # voluntary context switch
 
     def render(self):
-        """renders the entire window{"""
+        """renders the entire window"""
 
         self.frame_count += 1
-        if not self.help.is_active:
-            # fps also counts rendering time
-            # pause fps while help is active
-            self.monitor.tick()
-
-        # populate widgets
-        self.fps.set_fps(self.monitor.fps)
-        self.flags.set_flag_counts(config.mine_count - self.board.flag_count())
-        if not self.game_over and not self.help.is_active:
-            time_taken = datetime.datetime.now() - self.time_started
-            minute, second = divmod(time_taken.seconds, 60)
-            msec = str(time_taken.microseconds).zfill(2)[:2]
-            self.time_taken = f'{minute:0>2}:{second:0>2}.{msec}'
-            self.timer.set_text(self.time_taken)
-
         self.window.erase()
-        self.calc_widget_locations()
-        try:
-            self.window.addch(self.mouse_y, self.mouse_x, curses.ACS_DIAMOND)
-        except curses.error:
-            pass  # sometimes mouse fly around and that's ok
 
-        # check winning condition
-        if not self.game_over and self.board.check_win():
-            self.status.status = '😎'
-            self.game_over = True
-            self.board.reveal_all()
+        winh, winw = self.window.getmaxyx()
+        if winh < config.min_height or winw < config.min_width:
+            self.addstr(3, 3, "Insufficient screen space", curses.color_pair(UI_ERROR))
+            if winh < config.min_height:
+                self.addstr(4, 3, f"{config.min_height - winh} more rows required", curses.color_pair(UI_ERROR))
+            if winw < config.min_width:
+                self.addstr(4 + (winh < config.min_height), 3, f"{config.min_width - winw} more columns required",
+                            curses.color_pair(UI_ERROR))
+            self.addstr(5 + (winh < config.min_height) + (winw<config.min_width), 3, f"Press Ctrl-C to exit",
+                        curses.color_pair(UI_ERROR))
+        else:
+            if not self.help.is_active:
+                # fps also counts rendering time
+                # pause fps while help is active
+                self.monitor.tick()
 
-        for w in self.subwidgets:
-            if self.help.is_active and isinstance(w, GridWidget):
-                # Hide the grid while help is active
-                continue
-            w.render()
+            # populate widgets
+            self.fps.set_fps(self.monitor.fps)
+            self.flags.set_flag_counts(config.mine_count - self.board.flag_count())
+            if not self.game_over and not self.help.is_active:
+                time_taken = datetime.datetime.now() - self.time_started
+                minute, second = divmod(time_taken.seconds, 60)
+                msec = str(time_taken.microseconds).zfill(2)[:2]
+                self.time_taken = f'{minute:0>2}:{second:0>2}.{msec}'
+                self.timer.set_text(self.time_taken)
+
+            self.calc_widget_locations()
+
+            try:
+                self.window.addch(self.mouse_y, self.mouse_x, curses.ACS_DIAMOND)
+            except curses.error:
+                pass  # sometimes mouse fly around and that's ok
+
+            # check winning condition
+            if not self.game_over and self.board.check_win():
+                self.status.status = '😎'
+                self.game_over = True
+                self.board.reveal_all()
+
+            for w in self.subwidgets:
+                if self.help.is_active and isinstance(w, GridWidget):
+                    # Hide the grid while help is active
+                    continue
+                w.render()
 
         # overwrite all other widgets
         self.paint_window()
@@ -1018,6 +1048,7 @@ def main():
         curses.init_pair(DEFAULT, FG, BG)
         curses.init_pair(UI_HIGHLIGHT, UI_HIGHLIGHT_FG, UI_HIGHLIGHT_BG)
         curses.init_pair(UI_ALT_HIGHLIGHT, UI_HIGHLIGHT_FG, UI_ALT_HIGHLIGHT_BG)
+        curses.init_pair(UI_ERROR, UI_ERROR_FG, BG)
         curses.init_pair(SYSTEM_DEFAULT, -1, -1)
         for i in range(9):
             curses.init_pair(cell_color(i, True), VALUES[i], UI_HIGHLIGHT_BG)
